// SPDX-FileCopyrightText: no
// SPDX-License-Identifier: CC0-1.0
#ifndef CALAMARES_VERSION_H
#define CALAMARES_VERSION_H

#define CALAMARES_ORGANIZATION_NAME "Calamares"
#define CALAMARES_ORGANIZATION_DOMAIN "github.com/calamares"
#define CALAMARES_APPLICATION_NAME "Calamares"
#define CALAMARES_VERSION "3.3.10"
#define CALAMARES_VERSION_SHORT "3.3.10"

#define CALAMARES_VERSION_MAJOR "3"
#define CALAMARES_VERSION_MINOR "3"
#define CALAMARES_VERSION_PATCH "10"
#define CALAMARES_VERSION_RC "1"

#endif  // CALAMARES_VERSION_H
